//**************************************************************************
//   CC430F613x Demo - USCI_A0, 115200 UART Echo ISR, DCO SMCLK
//
//   Description: Echo a received character, RX ISR used. Normal mode is LPM0.
//   USCI_A0 RX interrupt triggers TX Echo.
//   Baud rate divider with 1048576hz = 1048576/115200 = ~9.1 (009h|01h)
//   ACLK = REFO = ~32768Hz, MCLK = SMCLK = default DCO = 32 x ACLK = 1048576Hz
//   See User Guide for baud rate divider table
//
//                 CC430F6137
//             -----------------
//         /|\|                 |
//          | |                 |
//          --|RST              |
//            |                 |
//            |     P1.6/UCA0TXD|------------>
//            |                 | 115200 - 8N1
//            |     P1.5/UCA0RXD|<------------
//
//   M Morales
//   Texas Instruments Inc.
//   April 2009
//   Built with CCE Version: 3.2.2 and IAR Embedded Workbench Version: 4.11B
//******************************************************************************
//this is the SECOND code (jaune 5&6)

//#include <msp430.h>
#include <cc430f5137.h>
#include <stdint.h>
#define UART_9600
#include <string.h>
//#define UART_115200

void uart_init(void)
{
  
  
  // WDTCTL = WDTPW + WDTHOLD;		// Stop WDT

#if 0
  // ======================
  // Port Mapping (now done in ports.c)
  // not really required because we use default pin
  PMAPPWD = 0x02D52;			// Get write-access to port mapping regs  
  P1MAP5 = PM_UCA0RXD;          // Map UCA0RXD output to P1.5 
  P1MAP6 = PM_UCA0TXD;          // Map UCA0TXD output to P1.6 
  PMAPPWD = 0;                  // Lock port mapping registers 
  // ======================
#endif
 

  P1DIR &= ~BIT5;		// Set P1.5 as RX input
  P1DIR |= BIT6;                // Set P1.6 as TX output
  P1SEL |= BIT5 + BIT6;         // Select P1.5 & P1.6 to UART function
#if 0
  P1DIR |= BIT6;                // Set P1.6 as TX output
  P1DIR &= ~BIT7;		// Set P1.7 as RX input
  P1SEL |= BIT7 + BIT6;         // Select P1.7 & P1.6 to UART function
#endif

  // ======================
  // UART configuration
  UCA0CTL1 |= UCSWRST;          // **Put state machine in reset**

#ifdef UART_115200
  UCA0CTL1 |= UCSSEL_2;         // SMCLK
  UCA0BR0 = 9;                  // 1MHz 115200 (see User's Guide)
  UCA0BR1 = 0;                  // 1MHz 115200
  UCA0MCTL |= UCBRS_1 + UCBRF_0;// Modulation UCBRSx=1, UCBRFx=0
#elif defined(UART_9600)
  UCA0CTL1 |= UCSSEL_1;         // CLK = ACLK
  UCA0BR0 = 0x03;               // 32k/9600 - 3.41
  UCA0BR1 = 0x00;               //
  UCA0MCTL = UCBRS_3 + UCBRF_0;   // Modulation UCBRSx=3, UCBRFx=0
#else
   #error Please select one of the supported baudrates.
#endif

  UCA0CTL1 &= ~UCSWRST;         // **Initialize USCI state machine**
  // ======================
  
  
  // ======================
  // Enable Interruption
  // UCA0IE |= UCRXIE | UCTXIE;             // Enable USCI_A0 TX & RX interrupt
  // ======================
}

// Function to send a byte on the serial line

void uart_send(unsigned char byte){
while (!(UCA0IFG & UCTXIFG));   // USCI_A0 TX buffer ready?
UCA0TXBUF = byte;             // TX -> parameter character
}

unsigned char uart_receive(void){
while (!(UCA0IFG & UCRXIFG));   // USCI_A0 RX buffer ready?
return UCA0RXBUF;             // TX -> parameter character
}

void uart_send_string(char *string){
int i;
for(i=0;i<strlen(string);i++) uart_send(string[i]);
uart_send('\n');
uart_send('\r');
}


// Echo back RXed character, confirm TX buffer is ready first
/*
#pragma vector=USCI_A0_VECTOR
__interrupt void USCI_A0_ISR(void)
{
  switch(UCA0IV)
  {
  case 0:break;                             // Vector 0 - no interrupt
  case 2:                                   // Vector 2 - RXIFG
    while (!(UCA0IFG&UCTXIFG));             // USCI_A0 TX buffer ready?
    UCA0TXBUF = UCA0RXBUF;                  // TX -> RXed character
    break;
  case 4:break;                             // Vector 4 - TXIFG
  default: break;
  }
}
*/
