#include <cc430f5137.h>
#include <msp430.h>
#include <stdbool.h>
#include <stddef.h>
#include <stdint.h>
#include <stdlib.h>
#include <stdio.h>

#include "serial.h"
#include "led_commands.h"
#include "ports.h"
#include "I2C.h"


void I2Cinit(slave_id) {
	
 P1SEL |= ( BIT2 | BIT3 );		// I2C uses UCB0, SCL on 1.2, SDA on 1.3
 
 UCB0CTL1 |= UCSWRST;
 UCB0CTL0 = UCMST + UCMODE_3 + UCSYNC;	// Mode master
 UCB0CTL1 = UCSSEL_2;			// + UCSWRST; // select SMCLK
 UCB0BR0 = 12;
 UCB0BR1 = 0;
 UCB0CTL1 &= ~UCSWRST;
	
 UCB0I2CSA = slave_id;			//ID of slave (out of reset block !)
}

int I2Cgetbyte(unsigned char reg_no) {
  UCB0CTL1 |= UCTXSTT + UCTR;   //Send Start message and then slave address
  while(!(UCB0IFG & UCTXIFG));	//Check that USCI is ready to transmit data
  UCB0TXBUF = reg_no;     	//Address of slave register to be read
  while((UCB0CTL1 & UCTXSTT) && !(UCB0IFG & UCNACKIFG));
				//Wait until the acknowlege message for slave address has been received     
				//or UCNACKIFG is set
  if(UCB0IFG & UCNACKIFG){
    UCB0CTL1 |= UCTXSTP;		//Abort communication
    while((UCB0CTL1 & UCTXSTP));	//Wait for Stop to be sent
    return -1;
  }
  while(!(UCB0IFG & UCTXIFG));	//Check that USCI has processed the data (probably useless)

  UCB0CTL1 &= ~UCTR;            //Sets the master as a receiver
  UCB0CTL1 |= UCTXSTT;          //Generate a repeated start (SR)     
  while((UCB0CTL1 & UCTXSTT) && !(UCB0IFG & UCNACKIFG));
				//Wait until the acknowlege message for slave address has been received     
				//or UCNACKIFG is set
  if(UCB0IFG & UCNACKIFG){
    UCB0CTL1 |= UCTXSTP;		//Abort communication
    while((UCB0CTL1 & UCTXSTP));	//Wait for Stop to be sent
    return -1;
  }

  UCB0CTL1 |= UCTXSTP;          //Send NACK and Stop message after the reading     
  while (!(UCB0IFG & UCRXIFG)); //Wait for slave data to be available
  unsigned char data = UCB0RXBUF;        
  while((UCB0CTL1 & UCTXSTP));	//Wait for Stop message to be transmitted
  return data;			//return communicated data
}

int I2Csetbyte(unsigned char reg_no,unsigned char value) {
  UCB0CTL1 |= UCTXSTT + UCTR;   //Send Start message and the slave address
  while(!(UCB0IFG & UCTXIFG));	//Check that USCI is ready to transmit data
  UCB0TXBUF = reg_no;     	//Address of slave register to be written    
  while((UCB0CTL1 & UCTXSTT) && !(UCB0IFG & UCNACKIFG));
				//Wait until the acknowlege message for slave address has been received     
				//or UCNACKIFG is set
  if(UCB0IFG & UCNACKIFG){
    UCB0CTL1 |= UCTXSTP;		//Abort communication
    while((UCB0CTL1 & UCTXSTP));	//Wait for Stop to be sent
    return -1;
  }
  
  while(!(UCB0IFG & UCTXIFG));	//Check that USCI has processed the data (probably useless) 
  UCB0TXBUF = value;     	//Send value to write into slave register
  while(!(UCB0IFG & UCTXIFG));	//Check that USCI has processed the data
  UCB0CTL1 |= UCTXSTP;          //Send Stop message after ACK has been received
  while(UCB0CTL1 & UCTXSTP);    //Wait until the Stop message has been sent
  return 0;
}