
#include <cc430f5137.h>
#include "serial.h"
#include <msp430.h>
#include <stdbool.h>
#include <stddef.h>
#include <stdint.h>
#include <stdlib.h>
#include <stdio.h>

/*
// The BMP180's I2C addresses are 0xEE for write, 0xEF for read
enum {
	AC1,
	AC2,
	AC3,
	AC4,
	AC5,
	AC6,
	B1,
	B2,
	MB,
	MC,
	MD,
	reg_count
};
typedef struct BMP {
	// These are all the internal calibration constants
	int16_t regs[reg_count + 1];
	// Over-sampling setting
	uint8_t oss;
	// Temperature and pressure data
	uint16_t therm;
	uint16_t pressure;
	// I2C send and receive data
	uint8_t sendData;
	uint8_t recvData;

} BMP_t;

BMP_t sensor, *pSensor = &sensor;
*/

void I2Cinit(slave_id) {
	
 P1SEL |= ( BIT2 | BIT3 );		// I2C uses UCB0, SCL on 1.2, SDA on 1.3
 
 UCB0CTL1 |= UCSWRST;
 UCB0CTL0 = UCMST + UCMODE_3 + UCSYNC;	// Mode master
 UCB0CTL1 = UCSSEL_2			// + UCSWRST; // select SMCLK
 UCB0BR0 = 12;
 UCB0BR1 = 0;
 UCB0CTL1 &= ~UCSWRST;
	
 UCB0I2CSA = slave_id;			//ID of slave (out of reset block ?)
}

unsigned char I2Cgetbyte(unsigned char reg_no) {
  UCB0CTL1 |= UCTXSTT + UCTR;   //Send Start message and then slave address
  while(!(UCB0IFG & UCTXIFG));	//Check that USCI is ready to transmit data
  UCB0TXBUF = reg_no;     	//Address of slave register to be read
  while(UCB0CTL1 & UCTXSTT);    //Wait until the acknowlege message for slave address has been received     
				//We should also check that UCNACKIFG is not set
  while(!(UCB0IFG & UCTXIFG));	//Check that USCI has processed the data (? probably useless)

  UCB0CTL1 &= ~UCTR;            //Sets the master as a receiver
  UCB0CTL1 |= UCTXSTT;          //Generate a repeated start     
  while(UCB0CTL1 & UCTXSTT);    //Wait until the acknowlege message for slave address has been received 
				//We should also check that UCNACKIFG is not set

  UCB0CTL1 |= UCTXSTP;          //Send NACK and Stop message after the reading     
  while (!(UCB0IFG & UCRXIFG)); //wait for receive to be done
  unsigned char data = UCB0RXBUF;//character variabla for received data        
  while((UCB0CTL1 & UCTXSTP));	//wait for transmit to be done
  return data;			//return communicated data
}

void I2Csetbyte(unsigned char reg_no,unsigned char value) {
  UCB0CTL1 |= UCTXSTT + UCTR;   //Send Start message and the slave address
  while(!(UCB0IFG & UCTXIFG));	//Check that USCI is ready to transmit data
  UCB0TXBUF = reg_no;     	//Address of slave register to be written    
  while(UCB0CTL1 & UCTXSTT);    //Wait until the acknowlege message for slave address has been received     
				//We should also check that UCNACKIFG is not set
  while(!(UCB0IFG & UCTXIFG));	//Check that USCI has processed the data (? probably useless) 
  UCB0TXBUF = value;     	//Send value to write into slave register
  while(!(UCB0IFG & UCTXIFG));	//Check that USCI has processed the data
  UCB0CTL1 |= UCTXSTP;          //Send Stop message after ACK has been received
  while(UCB0CTL1 & UCTXSTP);    //Wait until the Stop message has been sent
}

/*
void calcResult(void) {
      // not implemented yet
}
*/

int main(void) {
	char output[32];
	WDTCTL = WDTPW | WDTHOLD;		// Stop watchdog timer
	uart_init();				//serial communication with minicom
	
	I2Cinit(0x1d);				//Initialize slave (accelerometer) address
	I2Csetbyte(0x2a,0x01);			//set awake mode
	while(1){
	  int ax,ay,az;
	  ax=I2Cgetbyte(0x1);			//value for x-achsis. 0x1 is 7 bit. 0x2 10 bit
	  ay=I2Cgetbyte(0x3);			//value for y-achsis. 0x1 is 7 bit. 0x2 10 bit
	  az=I2Cgetbyte(0x5);			//value for z-achsis. 0x1 is 7 bit. 0x2 10 bit
	  sprintf(output,"%02x %02x %02x",ax,ay,az);//change value to hex
	  uart_send_string(output);		//send values to minicom
	  __delay_cycles(500000);		//wait
	}

	return 0;

}


//
/*
#include "msp430.h"
#include <cc430f5137.h>
#include <stdint.h>
#include "serial.h"
#include "ports.h"

#define I2C_SLAVE_ID	0x1D
#define I2C_ACCEL_REG	0x1B
#define I2C_PRESCALER	160 // pour un CPU à 16Mhz => I2C a 100Khz

void main(void)
{
WDTCTL = WDTPW + WDTHOLD; 			// Stopping WDT. Otherwise the device goes in standby mode if the time between two actions is too long.
uart_init();					//serial communication. sends string to minicom

//1. Set 
//2. Initialize all USCI registers
//3. Configure ports.
//4. Clear UCSWRST via software 
//5. Enable interrupts (optional).

//1
UCSWRST (BIS.B#UCSWRST,&UCxCTL1). 
//2
UCSWRST = 1.		
//3
  PMAPPWD = 0x02D52;                        // Get write-access to port mapping regs  
  P1MAP3 = PM_UCB0SDA;                      // Map UCB0SDA output to P1.3 
  P1MAP2 = PM_UCB0SCL;                      // Map UCB0SCL output to P1.2 
  PMAPPWD = 0;                              // Lock port mapping registers 
  
  P1SEL |= BIT2 + BIT3;                     // Select P1.2 & P1.3 to I2C function
//4
(BIC.B #UCSWRST,&UCxCTL1).
//5
_EINT(); // enable interrupts 


UCMODE_3=11;					// I2C Slave, 
UCSYNC=1; 					//synchronous mode

UCSLA10 = 0;					//7 bit adressing

UCBxI2CSA= I2C_SLAVE_ID;			// what for x

&= ~ UCTR  //??? clear UCTR 

UCB0CTL1 |= UCTXSTT;                    //setting UCTXSTT to generate a START condition
//transmit slave address
//ACK
UCB0CTL1 &= ~ UCTXSTT//UCTXSTT cleared
//ACK
//first data received from slave
//UCRXIFG flag is set.
//Data is received from the slave, as long as UCTXSTP or UCTXSTT is not set. 
//If UCBxRXBUF is not read, the master holds the bus during reception of the last data bit and until the UCBxRXBUF is read.
If the slave does not acknowledge the transmitted address, the not-acknowledge interrupt flag
UCNACKIFG is set. The master must react with either a STOP condition or a repeated START condition.
Setting the UCTXSTP bit generates a STOP condition. After setting UCTXSTP, a NACK followed by a
STOP condition is generated after reception of the data from the slave, or immediately if the USCI module
is currently waiting for UCBxRXBUF to be read.
If a master wants to receive a single byte only, the UCTXSTP bit must be set while the byte is being
received. For this case, the UCTXSTT may be polled to determine when it is cleared:




&= ~ //??? clear UCMST
*/