/*** USCI master library ************************************************************

In this file the usage of the USCI I2C master library without DMA support is 
shown. This library uses pointers to specify what data is to be sent. 

When calling the TI_USCI_I2C_receive or TI_USCI_I2C_transmit routines 
the number of bytes, which are to be transmitted or received have to be passed as 
well as a pointer to a data field, that contains(or stores) the data.

This code checks if there is a slave with address 0x50 is connected to the I2C
bus and if the slave device is present, bytes are received and transmitted.

Uli Kretzschmar
MSP430 Systems
Freising
*******************************************************************************/
/*
 * [Start] [slave address][ACK][Data][ACK][Data][ACK] [Stop]
*******************************************************************************/
#include "msp430.h"
#include <cc430f5137.h>
#include <stdint.h>
#include "TI_USCI_I2C_master_dma.h" //1. TI_USCI_I2C_master.c must be added to project.Supports I2C communication using the USCI module
#include "serial.h"
#include "ports.h"


#define I2C_SLAVE_ID	0x1D
#define I2C_ACCEL_REG	0x1B
#define I2C_PRESCALER	160 // pour un CPU à 16Mhz => I2C a 100Khz


void main(void)
{
WDTCTL = WDTPW + WDTHOLD; 			// Stopping WDT. Otherwise the device goes in standby mode if the time between two actions is too long.
uart_init();

_EINT(); // enable interrupts

//Start-initialization
TI_USCI_I2C_DMA_transmitinit(I2C_SLAVE_ID,I2C_PRESCALER);	// initialize USCI

//ACK
while ( TI_USCI_I2C_notready() ); 			// wait for bus to be free
//??????????????????????????????? array or register
unsigned char array[1]={I2C_ACCEL_REG};

//slave address
TI_USCI_I2C_DMA_transmit(1,array); 				// transmit the register address
//ACK
while ( TI_USCI_I2C_notready() ); 			// wait for bus to be free

//Data
TI_USCI_I2C_DMA_receiveinit(I2C_SLAVE_ID,I2C_PRESCALER);	//two values included. first is slave address second prescale factor for the baud rate

//ACK
while ( TI_USCI_I2C_notready() ); 			// wait for bus to be free
unsigned char value;

//Data
TI_USCI_I2C_DMA_receive(1, &value);

//Ack
while ( TI_USCI_I2C_notready() );

}




